@@ -23,11 +23,11 @@
 #' data(efc)
 #' x <- to_factor(efc$e42dep)
 #' str(x)
-#' table(x)
+#' frq(x)
 #'
 #' ref_lvl(x) <- 3
 #' str(x)
-#' table(x)
+#' frq(x)
 #'
 #' @export
 ref_lvl <- function(x, value = NULL) {
@@ -88,6 +88,5 @@ ref_lvl <- function(x, value = NULL) {
 
 #' @export
 `ref_lvl<-.default` <- function(x, value) {
-  x <- ref_lvl(x = x, value = value)
-  x
+  ref_lvl(x = x, value = value)
 }
